/*
 * @(#) Main.java 
 *
 * JOTM: Java Open Transaction Manager 
 *
 * This project is developed inside the ObjectWeb Consortium,
 * http://www.objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *
 * Contributor(s): Lutris Technologies, Guillaume Riviere
 *
 *
 * -------------------------------------------------------------------------
 * $Id: Main.java,v 1.14 2003/04/14 13:49:47 jmesnil Exp $
 * --------------------------------------------------------------------------
 */
package org.objectweb.jotm;

import java.io.PrintWriter;
import java.util.Enumeration;
import java.util.Hashtable;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.HelpFormatter;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;

import org.objectweb.jonas_timer.TraceTimer;
import org.objectweb.transaction.jta.TMService;

/**
 * This class is used to start JOTM as a standalone transaction manager.
 * 
 * @author  Christophe Ney - Lutris Technologies cney@batisseurs.com
 * Created on Feb 26, 2002
 * */
public class Main extends Thread {

    private static final String progname = "JOTM";

    // command line options
    private static Options cmdLineOptions = null;

    // verbose mode
    static boolean verbose = false;
    // debug mode
    static boolean debug = false;
    // default transaction timeout
    static int timeout = 0;
    // is transaction factory remote
    static boolean remote = false;
    // user transaction URL
    static String userTransactionName = null;
    // transaction manager URL
    static String transactionManagerName = null;
    // naming context
    static Context ictx = null;
    // logWriter
    static PrintWriter logWriter = new PrintWriter(System.out, true);

    // instance of JOTM
    private static TMService jotm;

    /**
     *  used as shutdown hook
     */
    public void run() {
        System.out.print("shutting down...");
        try {
            ictx.unbind(userTransactionName);
        } catch (Exception e) {
            // ignore
        }
        try {
            ictx.unbind(transactionManagerName);
        } catch (Exception e) {
            // ignore
        }
        // stop jotm
        try {
            jotm.stop();
        } catch (Exception e) {
            // ignore
        }
        logWriter.close();
    }

    public static void printHelp(Options cmdLineOptions) {
        HelpFormatter hf = new HelpFormatter();
        hf.printHelp("JOTM [options...]", cmdLineOptions);
    }

    private static void verbose(String msg) {
        if (verbose) {
            System.out.println(msg);
        }
    }

    private static void checkRegistryMessage() {
        System.err.println("Current JNDI settings are:");
        boolean found = false;
        try {
            Hashtable env = ictx.getEnvironment();
            for (Enumeration e = env.keys(); e.hasMoreElements();) {
                String key = (String) e.nextElement();
                System.err.println("- " + key + "=" + env.get(key));
                found = true;
            }
        } catch (NamingException e) {
            // ignore
        }
        if (!found) {
            System.err.println("JNDI properties are not set!");
        } else {
            System.err.println("Check that registry is running on a port matching JNDI properties");
        }
    }

	
    /**
     * Used to start JOTM from a command line interface.
     * 
     * @param args JOTM arguments
     */
    public static void main(String[] args) {

        cmdLineOptions = new Options();
        // option parameters are: short description (char), long description (String), has arguments (boolean), 
        //                        description (String), required (boolean), has multiple arguments (boolean)
        cmdLineOptions.addOption('d', "debug", false, "debug mode", false, false);
        cmdLineOptions.addOption('v', "verbose", false, "verbose mode", false, false);
        cmdLineOptions.addOption('h', "help", false, "print this message and exit", false, false);
        cmdLineOptions.addOption('m', "transaction_manager", true, "JNDI URL of the TransactionManager", false, false);
        cmdLineOptions.addOption('r', "remote", false, "lookup remote transaction factory", false, false);
        cmdLineOptions.addOption('t', "timeout", true, "default transaction timeout (in seconds)", false, false);
        cmdLineOptions.addOption('u', "user_transaction", true, "JNDI URL of the UserTransaction", false, false);

        CommandLine cmd = null;
        try {
            cmd = cmdLineOptions.parse(args, true);
        } catch (ParseException e) {
            System.err.println("\n" + e.getMessage());
            printHelp(cmdLineOptions);
            System.err.println();
            System.exit(1);
        }

        debug = cmd.hasOption('d');
        remote = cmd.hasOption('r');
        verbose = cmd.hasOption('v');
        if (cmd.hasOption('h')) {
            printHelp(cmdLineOptions);
            System.exit(1);
        }
        if (cmd.hasOption('m')) {
            transactionManagerName = cmd.getOptionValue('m');
        }
        if (cmd.hasOption('t')) {
            try {
                timeout = Integer.parseInt(cmd.getOptionValue('t'));
            } catch (NumberFormatException e) {
                System.err.println("\ntimeout is not a number");
                printHelp(cmdLineOptions);
                System.err.println();
                System.exit(1);
            }
        }
        if (cmd.hasOption('u')) {
            userTransactionName = cmd.getOptionValue('u');
        }

        verbose("UserTransaction Name =" + userTransactionName);
        verbose("TransactionManager Name =" + transactionManagerName);
        verbose("Transaction factory =" + (remote ? "remote" : "local"));
        verbose("Default transaction timeout =" + timeout);

        TraceTimer.setLogWriter(logWriter);
        TraceTimer.setVerbose(verbose);
        TraceTimer.setDebug(debug);

        try {
            // create an instance of JOTM 
            jotm = new Jotm(!remote, true);
        } catch (NamingException e) {
            System.out.println("unable to start JOTM!: "+ e.getMessage());
            System.exit(1);
        }

        // trap program termination
        Runtime.getRuntime().addShutdownHook(new Main());

        try {
            ictx = new InitialContext();
        } catch (NamingException e) {
            System.err.println("No initial context: " + e.getExplanation());
            e.printStackTrace();
            System.exit(1);
        }

        try {
            if (userTransactionName != null) {
                ictx.rebind(userTransactionName, jotm.getUserTransaction());
                System.out.println("UserTransaction object bound in JNDI with name " + userTransactionName);
            }
            if (transactionManagerName != null) {
                ictx.rebind(transactionManagerName, jotm.getTransactionManager());
                System.out.println("TransactionManager object bound in JNDI with name " + transactionManagerName);
            }
        } catch (NamingException e) {
            System.err.println("UserTransaction rebind failed :" + e.getExplanation());
            e.printStackTrace();
            checkRegistryMessage();
            System.exit(1);
        }

        System.out.print(progname + " is running...");
    }
}
